names = {
    "FCAS Providers": "NEM Registration and Exemption List.xls",
    "DISPATCHLOAD": "PUBLIC_DVD_DISPATCHLOAD",
    "NEXT_DAY_DISPATCHLOAD": "PUBLIC_NEXT_DAY_DISPATCHLOAD",
    "DUDETAILSUMMARY": "PUBLIC_DVD_DUDETAILSUMMARY",
    "PARTICIPANT": "PUBLIC_DVD_PARTICIPANT",
    "DUDETAIL": "PUBLIC_DVD_DUDETAIL",
    "DISPATCHCONSTRAINT": "PUBLIC_DVD_DISPATCHCONSTRAINT",
    "GENCONDATA": "PUBLIC_DVD_GENCONDATA",
    "DISPATCH_UNIT_SCADA": "PUBLIC_DVD_DISPATCH_UNIT_SCADA",
    "INTERMITTENT_GEN_SCADA": "PUBLIC_NEXT_DAY_INTERMITTENT_GEN_SCADA",
    "DISPATCHPRICE": "PUBLIC_DVD_DISPATCHPRICE",
    "SPDREGIONCONSTRAINT": "PUBLIC_DVD_SPDREGIONCONSTRAINT",
    "SPDCONNECTIONPOINTCONSTRAINT": "PUBLIC_DVD_SPDCONNECTIONPOINTCONSTRAINT",
    "SPDINTERCONNECTORCONSTRAINT": "PUBLIC_DVD_SPDINTERCONNECTORCONSTRAINT",
    "BIDPEROFFER_D": "PUBLIC_DVD_BIDPEROFFER_D",
    "DISPATCHINTERCONNECTORRES": "PUBLIC_DVD_DISPATCHINTERCONNECTORRES",
    "BIDDAYOFFER_D": "PUBLIC_DVD_BIDDAYOFFER_D",
    "DISPATCHREGIONSUM": "PUBLIC_DVD_DISPATCHREGIONSUM",
    "FCAS_4_SECOND": "FCAS",
    "ELEMENTS_FCAS_4_SECOND": "Elements_FCAS.csv",
    "VARIABLES_FCAS_4_SECOND": "Ancillary Services Market Causer Pays Variables File.csv",
    "Generators and Scheduled Loads": "NEM Registration and Exemption List.xls",
    "MNSP_INTERCONNECTOR": "PUBLIC_DVD_MNSP_INTERCONNECTOR",
    "MNSP_PEROFFER": "PUBLIC_DVD_MNSP_PEROFFER",
    "INTERCONNECTOR": "PUBLIC_DVD_INTERCONNECTOR",
    "INTERCONNECTORCONSTRAINT": "PUBLIC_DVD_INTERCONNECTORCONSTRAINT",
    "MNSP_DAYOFFER": "PUBLIC_DVD_MNSP_DAYOFFER",
    "LOSSMODEL": "PUBLIC_DVD_LOSSMODEL",
    "LOSSFACTORMODEL": "PUBLIC_DVD_LOSSFACTORMODEL",
    "FCAS_4s_SCADA_MAP": "",
    "PLANTSTATS": "",
    "TRADINGLOAD": "PUBLIC_DVD_TRADINGLOAD",
    "TRADINGPRICE": "PUBLIC_DVD_TRADINGPRICE",
    "TRADINGREGIONSUM": "PUBLIC_DVD_TRADINGREGIONSUM",
    "TRADINGINTERCONNECT": "PUBLIC_DVD_TRADINGINTERCONNECT",
    "MARKET_PRICE_THRESHOLDS": "PUBLIC_DVD_MARKET_PRICE_THRESHOLDS",
    "DAILY_REGION_SUMMARY": "PUBLIC_DAILY_REGION_SUMMARY",
    "ROOFTOP_PV_ACTUAL": "PUBLIC_DVD_ROOFTOP_PV_ACTUAL",
}

table_types = {
    "FCAS Providers": "STATICXL",
    "DISPATCHLOAD": "MMS",
    "NEXT_DAY_DISPATCHLOAD": "NEXT_DAY_DISPATCHLOAD",
    "INTERMITTENT_GEN_SCADA": "INTERMITTENT_GEN_SCADA",
    "DUDETAILSUMMARY": "MMS",
    "PARTICIPANT": "MMS",
    "DUDETAIL": "MMS",
    "DISPATCHCONSTRAINT": "MMS",
    "GENCONDATA": "MMS",
    "DISPATCH_UNIT_SCADA": "MMS",
    "DISPATCHPRICE": "MMS",
    "SPDREGIONCONSTRAINT": "MMS",
    "SPDCONNECTIONPOINTCONSTRAINT": "MMS",
    "SPDINTERCONNECTORCONSTRAINT": "MMS",
    "BIDPEROFFER_D": "BIDDING",
    "DISPATCHINTERCONNECTORRES": "MMS",
    "BIDDAYOFFER_D": "BIDDING",
    "DISPATCHREGIONSUM": "MMS",
    "FCAS_4_SECOND": "FCAS",
    "ELEMENTS_FCAS_4_SECOND": "STATIC",
    "VARIABLES_FCAS_4_SECOND": "STATIC",
    "Generators and Scheduled Loads": "STATICXL",
    "MNSP_INTERCONNECTOR": "MMS",
    "MNSP_PEROFFER": "MMS",
    "INTERCONNECTOR": "MMS",
    "INTERCONNECTORCONSTRAINT": "MMS",
    "MNSP_DAYOFFER": "MMS",
    "LOSSMODEL": "MMS",
    "LOSSFACTORMODEL": "MMS",
    "FCAS_4s_SCADA_MAP": "CUSTOM",
    "TRADINGLOAD": "MMS",
    "TRADINGPRICE": "MMS",
    "TRADINGREGIONSUM": "MMS",
    "TRADINGINTERCONNECT": "MMS",
    "MARKET_PRICE_THRESHOLDS": "MMS",
    "DAILY_REGION_SUMMARY": "DAILY_REGION_SUMMARY",
    "ROOFTOP_PV_ACTUAL": "MMS",
}

dynamic_tables = [
    table
    for table, type in table_types.items()
    if type in ["MMS", "BIDDING", "DAILY_REGION_SUMMARY", "NEXT_DAY_DISPATCHLOAD", "INTERMITTENT_GEN_SCADA", "FCAS"]
]

return_tables = list(names.keys())

display_as_AMEO = [
    "FCAS Providers",
    "DISPATCHLOAD",
    "NEXT_DAY_DISPATCHLOAD",
    "INTERMITTENT_GEN_SCADA",
    "DUDETAILSUMMARY",
    "PARTICIPANT",
    "DUDETAIL",
    "DISPATCHCONSTRAINT",
    "GENCONDATA",
    "DISPATCH_UNIT_SCADA",
    "DISPATCHPRICE",
    "SPDREGIONCONSTRAINT",
    "SPDCONNECTIONPOINTCONSTRAINT",
    "SPDINTERCONNECTORCONSTRAINT",
    "BIDPEROFFER_D",
    "DISPATCHINTERCONNECTORRES",
    "BIDDAYOFFER_D",
    "DISPATCHREGIONSUM",
    "FCAS_4_SECOND",
    "ELEMENTS_FCAS_4_SECOND",
    "VARIABLES_FCAS_4_SECOND",
    "Generators and Scheduled Loads",
    "TRADINGLOAD",
    "TRADINGPRICE",
    "TRADINGREGIONSUM",
    "TRADINGINTERCONNECT",
    "ROOFTOP_PV_ACTUAL",
]

display_as_Custom = ["FCAS_4s_SCADA_MAP", "PLANTSTATS"]

static_tables = [
    "ELEMENTS_FCAS_4_SECOND",
    "VARIABLES_FCAS_4_SECOND",
    "Generators and Scheduled Loads",
    "FCAS Providers",
]

nem_web_domain_url = "https://nemweb.com.au/"

static_table_url = {
    "ELEMENTS_FCAS_4_SECOND": "https://www.nemweb.com.au/Reports/Current/Causer_Pays_Elements/",
    "VARIABLES_FCAS_4_SECOND": "https://aemo.com.au/-/media/files/electricity/nem/settlements_and_payments/settlements/auction-reports/archive/ancillary-services-market-causer-pays-variables-file.csv",
    "Generators and Scheduled Loads": "https://www.aemo.com.au/-/media/Files/Electricity/NEM/Participant_Information/NEM-Registration-and-Exemption-List.xls",
    "_downloader.download_xl": "https://www.aemo.com.au/-/media/Files/Electricity/NEM/Participant_Information/NEM-Registration-and-Exemption-List.xls",
}

aemo_mms_url = "http://www.nemweb.com.au/Data_Archive/Wholesale_Electricity/MMSDM/{}/MMSDM_{}_{}/MMSDM_Historical_Data_SQLLoader/DATA/{}.zip"

current_data_page_urls = {
    "BIDDING": "Reports/Current/Bidmove_Complete/",
    "DAILY_REGION_SUMMARY": "/Reports/Current/Daily_Reports/",
    "NEXT_DAY_DISPATCHLOAD": "/Reports/Current/NEXT_DAY_DISPATCH/",
    "INTERMITTENT_GEN_SCADA": "/Reports/Current/Next_Day_Intermittent_Gen_Scada/"
}

fcas_4_url = "http://www.nemweb.com.au/Reports/Current/Causer_Pays/FCAS_{}{}{}{}.zip"

fcas_4_url_hist = "http://www.nemweb.com.au/Data_Archive/Wholesale_Electricity/FCAS_Causer_Pays/{}/FCAS_Causer_Pays_{}_{}/FCAS_{}{}{}{}.zip"

data_url = {
    "DISPATCHLOAD": "aemo_data_url",
    "DUDETAILSUMMARY": "aemo_data_url",
    "PARTICIPANT": "aemo_data_url",
    "DUDETAIL": "aemo_data_url",
    "DISPATCHCONSTRAINT": "aemo_data_url",
    "GENCONDATA": "aemo_data_url",
    "DISPATCH_UNIT_SCADA": "aemo_data_url",
    "DISPATCHPRICE": "aemo_data_url",
    "SPDREGIONCONSTRAINT": "aemo_data_url",
    "SPDCONNECTIONPOINTCONSTRAINT": "aemo_data_url",
    "SPDINTERCONNECTORCONSTRAINT": "aemo_data_url",
    "BIDPEROFFER_D": "aemo_data_url",
    "DISPATCHINTERCONNECTORRES": "aemo_data_url",
    "INTERCONNECTOR": "aemo_data_url",
    "INTERCONNECTORCONSTRAINT": "aemo_data_url",
    "MNSP_INTERCONNECTOR": "aemo_data_url",
    "BIDDAYOFFER_D": "aemo_data_url",
    "DISPATCHREGIONSUM": "aemo_data_url",
    "MNSP_DAYOFFER": "aemo_data_url",
    "MNSP_PEROFFER": "aemo_data_url",
    "LOSSMODEL": "aemo_data_url",
    "LOSSFACTORMODEL": "aemo_data_url",
    "DISPATCHCASESOLUTION": "aemo_data_url",
    "FCAS": "fcas_4_url",
    "TRADINGLOAD": "aemo_data_url",
    "TRADINGPRICE": "aemo_data_url",
    "TRADINGREGIONSUM": "aemo_data_url",
    "TRADINGINTERCONNECT": "aemo_data_url",
    "MARKET_PRICE_THRESHOLDS": "aemo_data_url",
    "ROOFTOP_PV_ACTUAL": "aemo_data_url",
}

filterable_cols = [
    "DUID",
    "REGIONID",
    "STATIONID",
    "PARTICIPANTID",
    "STARTTYPE",
    "SCHEDULE_TYPE",
    "GENCONID",
    "BIDTYPE",
    "VARIABLEID",
    "INTERVENTION",
    "DISPATCHMODE",
    "STARTTYPE",
    "CONNECTIONPOINTID",
    "DISPATCHTYPE",
    "CONSTRAINTID",
    "PREDISPATCH",
    "STPASA",
    "MTPASA",
    "LIMITTYPE",
    "STATIONNAME",
    "AGCFLAG",
    "INTERCONNECTORID",
    "NAME",
    "Fuel Source - Primary",
    "Fuel Source - Descriptor",
    "Technology Type - Primary",
    "Technology Type - Descriptor",
    "ELEMENTNUMBER",
    "MARKETNAME",
    "VARIABLENUMBER",
    "VARIABLETYPE",
    "MMSDESCRIPTOR",
    "ELEMENTTYPE",
    "Region",
    "Max Cap (MW)",
    "Min Enablement Level",
    "Max Enablement Level",
    "Max Lower Angle",
    "Max Upper Angle",
    "Bid Type",
    "TYPE",
]

table_columns = {
    "DISPATCHLOAD": [
        "SETTLEMENTDATE",
        "DUID",
        "INTERVENTION",
        "DISPATCHMODE",
        "AGCSTATUS",
        "INITIALMW",
        "TOTALCLEARED",
        "RAMPDOWNRATE",
        "RAMPUPRATE",
        "LOWER5MIN",
        "LOWER60SEC",
        "LOWER6SEC",
        "LOWER1SEC",
        "RAISE5MIN",
        "RAISE60SEC",
        "RAISE6SEC",
        "RAISE1SEC",
        "LOWERREG",
        "RAISEREG",
        "SEMIDISPATCHCAP",
        "AVAILABILITY",
        "RAISEREGENABLEMENTMAX",
        "RAISEREGENABLEMENTMIN",
        "LOWERREGENABLEMENTMAX",
        "LOWERREGENABLEMENTMIN",
    ],
    "NEXT_DAY_DISPATCHLOAD": [
        "SETTLEMENTDATE",
        "DUID",
        "INTERVENTION",
        "DISPATCHMODE",
        "AGCSTATUS",
        "INITIALMW",
        "TOTALCLEARED",
        "RAMPDOWNRATE",
        "RAMPUPRATE",
        "LOWER5MIN",
        "LOWER60SEC",
        "LOWER6SEC",
        "LOWER1SEC",
        "RAISE5MIN",
        "RAISE60SEC",
        "RAISE6SEC",
        "RAISE1SEC",
        "LOWERREG",
        "RAISEREG",
        "SEMIDISPATCHCAP",
        "AVAILABILITY",
        "RAISEREGENABLEMENTMAX",
        "RAISEREGENABLEMENTMIN",
        "LOWERREGENABLEMENTMAX",
        "LOWERREGENABLEMENTMIN",
    ],
    "INTERMITTENT_GEN_SCADA": [
        "RUN_DATETIME",
        "DUID",
        "SCADA_TYPE",
        "SCADA_VALUE",
        "SCADA_QUALITY"
    ],
    "TRADINGLOAD": [
        "SETTLEMENTDATE",
        "DUID",
        "INITIALMW",
        "TOTALCLEARED",
        "RAMPDOWNRATE",
        "RAMPUPRATE",
        "LOWER5MIN",
        "LOWER60SEC",
        "LOWER6SEC",
        "RAISE5MIN",
        "RAISE60SEC",
        "RAISE6SEC",
        "LOWERREG",
        "RAISEREG",
        "SEMIDISPATCHCAP",
        "AVAILABILITY",
    ],
    "TRADINGPRICE": [
        "SETTLEMENTDATE",
        "REGIONID",
        "RRP",
        "RAISE6SECRRP",
        "RAISE60SECRRP",
        "RAISE5MINRRP",
        "RAISEREGRRP",
        "LOWER6SECRRP",
        "LOWER60SECRRP",
        "LOWER5MINRRP",
        "LOWERREGRRP",
        "PRICE_STATUS",
    ],
    "TRADINGREGIONSUM": [
        "SETTLEMENTDATE",
        "REGIONID",
        "TOTALDEMAND",
        "AVAILABLEGENERATION",
        "AVAILABLELOAD",
        "DEMANDFORECAST",
        "DISPATCHABLEGENERATION",
        "DISPATCHABLELOAD",
        "NETINTERCHANGE",
        "EXCESSGENERATION",
        "LOWER5MINLOCALDISPATCH",
        "LOWER60SECLOCALDISPATCH",
        "LOWER6SECLOCALDISPATCH",
        "RAISE5MINLOCALDISPATCH",
        "RAISE60SECLOCALDISPATCH",
        "RAISE6SECLOCALDISPATCH",
        "LOWERREGLOCALDISPATCH",
        "RAISEREGLOCALDISPATCH",
        "INITIALSUPPLY",
        "CLEAREDSUPPLY",
        "TOTALINTERMITTENTGENERATION",
        "DEMAND_AND_NONSCHEDGEN",
        "UIGF",
    ],
    "TRADINGINTERCONNECT": [
        "SETTLEMENTDATE",
        "INTERCONNECTORID",
        "MWFLOW",
        "METEREDMWFLOW",
        "MWLOSSES",
    ],
    "DUDETAILSUMMARY": [
        "DUID",
        "START_DATE",
        "END_DATE",
        "DISPATCHTYPE",
        "CONNECTIONPOINTID",
        "REGIONID",
        "STATIONID",
        "PARTICIPANTID",
        "LASTCHANGED",
        "TRANSMISSIONLOSSFACTOR",
        "STARTTYPE",
        "DISTRIBUTIONLOSSFACTOR",
        "SCHEDULE_TYPE",
        "MAX_RAMP_RATE_UP",
        "MAX_RAMP_RATE_DOWN",
    ],
    "PARTICIPANT": [
        "PARTICIPANTID",
        "PARTICIPANTCLASSID",
        "NAME",
        "LASTCHANGED"
    ],
    "DISPATCHCONSTRAINT": [
        "SETTLEMENTDATE",
        "RUNNO",
        "CONSTRAINTID",
        "INTERVENTION",
        "RHS",
        "MARGINALVALUE",
        "VIOLATIONDEGREE",
        "LASTCHANGED",
        "GENCONID_EFFECTIVEDATE",
        "GENCONID_VERSIONNO",
        "LHS",
        "DISPATCHINTERVAL",
    ],
    "GENCONDATA": [
        "GENCONID",
        "EFFECTIVEDATE",
        "VERSIONNO",
        "CONSTRAINTTYPE",
        "CONSTRAINTVALUE",
        "DESCRIPTION",
        "GENERICCONSTRAINTWEIGHT",
        "LASTCHANGED",
        "DISPATCH",
        "PREDISPATCH",
        "STPASA",
        "MTPASA",
        "LIMITTYPE",
        "REASON",
    ],
    "DISPATCH_UNIT_SCADA": [
        "SETTLEMENTDATE",
        "DUID",
        "SCADAVALUE"],
    "DUDETAIL": [
        "EFFECTIVEDATE",
        "DUID",
        "VERSIONNO",
        "CONNECTIONPOINTID",
        "REGISTEREDCAPACITY",
        "AGCCAPABILITY",
        "DISPATCHTYPE",
        "MAXCAPACITY",
        "STARTTYPE",
        "NORMALLYONFLAG",
        "LASTCHANGED",
    ],
    "DISPATCHPRICE": [
        "SETTLEMENTDATE",
        "REGIONID",
        "INTERVENTION",
        "RRP",
        "RAISE6SECRRP",
        "RAISE60SECRRP",
        "RAISE5MINRRP",
        "RAISEREGRRP",
        "LOWER6SECRRP",
        "LOWER60SECRRP",
        "LOWER5MINRRP",
        "LOWERREGRRP",
        "PRICE_STATUS",
    ],
    "SPDREGIONCONSTRAINT": [
        "REGIONID",
        "EFFECTIVEDATE",
        "VERSIONNO",
        "GENCONID",
        "FACTOR",
        "LASTCHANGED",
        "BIDTYPE",
    ],
    "SPDCONNECTIONPOINTCONSTRAINT": [
        "CONNECTIONPOINTID",
        "EFFECTIVEDATE",
        "VERSIONNO",
        "GENCONID",
        "FACTOR",
        "BIDTYPE",
        "LASTCHANGED",
    ],
    "SPDINTERCONNECTORCONSTRAINT": [
        "INTERCONNECTORID",
        "EFFECTIVEDATE",
        "VERSIONNO",
        "GENCONID",
        "FACTOR",
        "LASTCHANGED",
    ],
    "BIDPEROFFER_D": [
        "DUID",
        "BANDAVAIL1",
        "BANDAVAIL2",
        "BANDAVAIL3",
        "BANDAVAIL4",
        "BANDAVAIL5",
        "BANDAVAIL6",
        "BANDAVAIL7",
        "BANDAVAIL8",
        "BANDAVAIL9",
        "BANDAVAIL10",
        "MAXAVAIL",
        "BIDTYPE",
        "DIRECTION",
        "SETTLEMENTDATE",
        "ENABLEMENTMIN",
        "ENABLEMENTMAX",
        "LOWBREAKPOINT",
        "HIGHBREAKPOINT",
        "INTERVAL_DATETIME",
        "OFFERDATE",
    ],
    "DISPATCHINTERCONNECTORRES": [
        "SETTLEMENTDATE",
        "INTERCONNECTORID",
        "DISPATCHINTERVAL",
        "INTERVENTION",
        "MWFLOW",
        "METEREDMWFLOW",
        "MWLOSSES",
    ],
    "INTERCONNECTOR": ["INTERCONNECTORID", "REGIONFROM", "REGIONTO", "LASTCHANGED"],
    "INTERCONNECTORCONSTRAINT": [
        "INTERCONNECTORID",
        "FROMREGIONLOSSSHARE",
        "EFFECTIVEDATE",
        "VERSIONNO",
        "LOSSCONSTANT",
        "LOSSFLOWCOEFFICIENT",
        "ICTYPE",
    ],
    "MNSP_INTERCONNECTOR": [
        "INTERCONNECTORID",
        "LINKID",
        "FROMREGION",
        "TOREGION",
        "MAXCAPACITY",
        "FROM_REGION_TLF",
        "TO_REGION_TLF",
        "LHSFACTOR",
        "EFFECTIVEDATE",
        "VERSIONNO",
    ],
    "BIDDAYOFFER_D": [
        "SETTLEMENTDATE",
        "DUID",
        "BIDTYPE",
        "DIRECTION",
        "OFFERDATE",
        "VERSIONNO",
        "PRICEBAND1",
        "PRICEBAND2",
        "PRICEBAND3",
        "PRICEBAND4",
        "PRICEBAND5",
        "PRICEBAND6",
        "PRICEBAND7",
        "PRICEBAND8",
        "PRICEBAND9",
        "PRICEBAND10",
        "T1",
        "T2",
        "T3",
        "T4",
        "MINIMUMLOAD",
    ],
    "DISPATCHREGIONSUM": [
        "SETTLEMENTDATE",
        "REGIONID",
        "DISPATCHINTERVAL",
        "INTERVENTION",
        "TOTALDEMAND",
        "AVAILABLEGENERATION",
        "AVAILABLELOAD",
        "DEMANDFORECAST",
        "DISPATCHABLEGENERATION",
        "DISPATCHABLELOAD",
        "NETINTERCHANGE",
        "EXCESSGENERATION",
        "LOWER5MINLOCALDISPATCH",
        "LOWER60SECLOCALDISPATCH",
        "LOWER6SECLOCALDISPATCH",
        "RAISE5MINLOCALDISPATCH",
        "RAISE60SECLOCALDISPATCH",
        "RAISE6SECLOCALDISPATCH",
        "LOWERREGLOCALDISPATCH",
        "RAISEREGLOCALDISPATCH",
        "INITIALSUPPLY",
        "CLEAREDSUPPLY",
        "TOTALINTERMITTENTGENERATION",
        "DEMAND_AND_NONSCHEDGEN",
        "UIGF",
        "SEMISCHEDULE_CLEAREDMW",
        "SEMISCHEDULE_COMPLIANCEMW",
    ],
    "MNSP_PEROFFER": [
        "SETTLEMENTDATE",
        "OFFERDATE",
        "VERSIONNO",
        "PARTICIPANTID",
        "LINKID",
        "PERIODID",
        "BANDAVAIL1",
        "BANDAVAIL2",
        "BANDAVAIL3",
        "BANDAVAIL4",
        "BANDAVAIL5",
        "BANDAVAIL6",
        "BANDAVAIL7",
        "BANDAVAIL8",
        "BANDAVAIL9",
        "BANDAVAIL10",
    ],
    "MNSP_DAYOFFER": [
        "SETTLEMENTDATE",
        "OFFERDATE",
        "VERSIONNO",
        "PARTICIPANTID",
        "LINKID",
        "PERIODID",
        "PRICEBAND1",
        "PRICEBAND2",
        "PRICEBAND3",
        "PRICEBAND4",
        "PRICEBAND5",
        "PRICEBAND6",
        "PRICEBAND7",
        "PRICEBAND8",
        "PRICEBAND9",
        "PRICEBAND10",
    ],
    "LOSSMODEL": [
        "EFFECTIVEDATE",
        "VERSIONNO",
        "INTERCONNECTORID",
        "LOSSSEGMENT",
        "MWBREAKPOINT",
    ],
    "LOSSFACTORMODEL": [
        "EFFECTIVEDATE",
        "VERSIONNO",
        "INTERCONNECTORID",
        "REGIONID",
        "DEMANDCOEFFICIENT",
    ],
    "DISPATCHCASESOLUTION": ["SETTLEMENTDATE", "TOTALOBJECTIVE"],
    "FCAS_4_SECOND": [
        "TIMESTAMP",
        "ELEMENTNUMBER",
        "VARIABLENUMBER",
        "VALUE",
        "VALUEQUALITY",
    ],
    "ELEMENTS_FCAS_4_SECOND": [
        "ELEMENTNUMBER",
        "EMSNAME",
        "ELEMENTTYPE",
        "MMSDESCRIPTOR",
    ],
    "VARIABLES_FCAS_4_SECOND": ["VARIABLENUMBER", "VARIABLETYPE"],
    "Generators and Scheduled Loads": [
        "Participant",
        "Station Name",
        "Region",
        "Dispatch Type",
        "Category",
        "Classification",
        "Fuel Source - Primary",
        "Fuel Source - Descriptor",
        "Technology Type - Primary",
        "Technology Type - Descriptor",
        "Aggregation",
        "DUID",
        "Reg Cap (MW)",
    ],
    "FCAS Providers": [
        "Participant",
        "Station Name",
        "Region",
        "DUID",
        "Bid Type",
        "Max Cap (MW)",
        "Min Enablement Level",
        "Max Enablement Level",
        "Max Lower Angle",
        "Max Upper Angle",
    ],
    "FCAS_4s_SCADA_MAP": ["ELEMENTNUMBER", "MARKETNAME", "ERROR"],
    "PLANTSTATS": [
        "Month",
        "DUID",
        "CapacityFactor",
        "Volume",
        "TRADING_VWAP",
        "DISPATCH_VWAP",
        "NodalPeakCapacityFactor",
        "Nodal90thPercentileCapacityFactor",
    ],
    "MARKET_PRICE_THRESHOLDS": [
        "EFFECTIVEDATE",
        "VERSIONNO",
        "VOLL",
        "MARKETPRICEFLOOR",
    ],
    "DAILY_REGION_SUMMARY": [
        "SETTLEMENTDATE",
        "RUNNO",
        "REGIONID",
        "INTERVENTION",
        "RRP",
        "EEP",
        "ROP",
        "APCFLAG",
        "MARKETSUSPENDEDFLAG",
        "TOTALDEMAND",
        "DEMANDFORECAST",
        "DISPATCHABLEGENERATION",
        "DISPATCHABLELOAD",
        "NETINTERCHANGE"
    ],
    "ROOFTOP_PV_ACTUAL": [
        "INTERVAL_DATETIME",
        "REGIONID",
        "POWER",
        "QI",
        "TYPE",
        "LASTCHANGED"
    ],
}

table_primary_keys = {
    "DISPATCHCONSTRAINT": [
        "CONSTRAINTID",
        "GENCONID_EFFECTIVEDATE",
        "GENCONID_VERSIONNO",
        "SETTLEMENTDATE",
        "INTERVENTION",
    ],
    "DUDETAILSUMMARY": ["DUID", "START_DATE", "END_DATE"],
    "PARTICIPANT": ["PARTICIPANTID", "LASTCHANGED"],
    "STATION": ["STATIONID"],
    "DUDETAIL": ["EFFECTIVEDATE", "DUID", "VERSIONNO"],
    "SPDREGIONCONSTRAINT": [
        "EFFECTIVEDATE",
        "GENCONID",
        "REGIONID",
        "VERSIONNO",
        "BIDTYPE",
    ],
    "SPDCONNECTIONPOINTCONSTRAINT": [
        "EFFECTIVEDATE",
        "GENCONID",
        "CONNECTIONPOINTID",
        "VERSIONNO",
        "BIDTYPE",
    ],
    "SPDINTERCONNECTORCONSTRAINT": [
        "EFFECTIVEDATE",
        "GENCONID",
        "INTERCONNECTORID",
        "VERSIONNO",
    ],
    "GENCONDATA": ["GENCONID", "EFFECTIVEDATE", "VERSIONNO"],
    "MNSP_PEROFFER": [
        "SETTLEMENTDATE",
        "OFFERDATE",
        "VERSIONNO",
        "PARTICIPANTID",
        "LINKID",
        "PERIODID",
    ],
    "MNSP_DAYOFFER": [
        "SETTLEMENTDATE",
        "OFFERDATE",
        "VERSIONNO",
        "PARTICIPANTID",
        "LINKID",
    ],
    "INTERCONNECTORCONSTRAINT": ["EFFECTIVEDATE", "INTERCONNECTORID", "VERSIONNO"],
    "MNSP_INTERCONNECTOR": ["EFFECTIVEDATE", "LINKID", "VERSIONNO"],
    "LOSSMODEL": ["EFFECTIVEDATE", "INTERCONNECTORID", "LOSSSEGMENT", "VERSIONNO"],
    "LOSSFACTORMODEL": ["EFFECTIVEDATE", "INTERCONNECTORID", "REGIONID", "VERSIONNO"],
    "BIDPEROFFER_D": [
        "BIDTYPE",
        "DUID",
        "OFFERDATE",
        "INTERVAL_DATETIME",
        "SETTLEMENTDATE",
        "DIRECTION"
    ],
    "DISPATCHINTERCONNECTORRES": [
        "DISPATCHINTERVAL",
        "INTERCONNECTORID",
        "INTERVENTION",
        "SETTLEMENTDATE",
    ],
    "INTERCONNECTOR": ["INTERCONNECTORID"],
    "DISPATCHPRICE": ["INTERVENTION", "REGIONID", "SETTLEMENTDATE"],
    "BIDDAYOFFER_D": ["BIDTYPE", "DUID", "SETTLEMENTDATE", "DIRECTION"],
    "DISPATCHREGIONSUM": [
        "DISPATCHINTERVAL",
        "INTERVENTION",
        "REGIONID",
        "SETTLEMENTDATE",
    ],
    "DISPATCHLOAD": ["SETTLEMENTDATE", "INTERVENTION", "DUID"],
    "NEXT_DAY_DISPATCHLOAD": ["SETTLEMENTDATE", "INTERVENTION", "DUID"],
    "INTERMITTENT_GEN_SCADA": [
        "RUN_DATETIME",
        "DUID",
        "SCADA_TYPE",
    ],
    "DISPATCH_UNIT_SCADA": ["SETTLEMENTDATE", "DUID"],
    "FCAS_4_SECOND": ["TIMESTAMP", "ELEMENTNUMBER", "VARIABLENUMBER"],
    "ELEMENTS_FCAS_4_SECOND": ["ELEMENTNUMBER"],
    "VARIABLES_FCAS_4_SECOND": ["VARIABLENUMBER", "VARIABLETYPE"],
    "Generators and Scheduled Loads": ["DUID"],
    "FCAS Providers": ["DUID", "Bid Type"],
    "FCAS_4s_SCADA_MAP": ["ELEMENTNUMBER", "MARKETNAME"],
    "TRADINGLOAD": ["SETTLEMENTDATE", "DUID"],
    "TRADINGPRICE": ["SETTLEMENTDATE", "REGIONID"],
    "TRADINGREGIONSUM": ["SETTLEMENTDATE", "REGIONID"],
    "TRADINGINTERCONNECT": ["SETTLEMENTDATE", "INTERCONNECTORID"],
    "PLANTSTATS": ["Month", "DUID"],
    "MARKET_PRICE_THRESHOLDS": ["EFFECTIVEDATE", "VERSIONNO"],
    "DAILY_REGION_SUMMARY": [
        "SETTLEMENTDATE",
        "INTERVENTION",
        "REGIONID",
    ],
    "ROOFTOP_PV_ACTUAL": [
        "INTERVAL_DATETIME",
        "REGIONID",
        "TYPE",
    ],
}

effective_date_group_col = {
    "SPDREGIONCONSTRAINT": ["GENCONID"],
    "SPDCONNECTIONPOINTCONSTRAINT": ["GENCONID"],
    "SPDINTERCONNECTORCONSTRAINT": ["GENCONID"],
    "GENCONDATA": ["GENCONID"],
    "MNSP_INTERCONNECTOR": ["INTERCONNECTORID"],
    "INTERCONNECTORCONSTRAINT": ["INTERCONNECTORID"],
    "INTERCONNECTOR": ["INTERCONNECTORID"],
    "LOSSMODEL": ["INTERCONNECTORID"],
    "LOSSFACTORMODEL": ["INTERCONNECTORID"],
    "DUDETAILSUMMARY": ["DUID"],
    "PARTICIPANT": ["PARTICIPANTID"],
    "MNSP_PEROFFER": ["LINKID"],
    "MNSP_DAYOFFER": ["LINKID"],
    "DUDETAIL": ["DUID"],
    "MARKET_PRICE_THRESHOLDS": [],
}

primary_date_columns = {
    "DISPATCHLOAD": "SETTLEMENTDATE",
    "NEXT_DAY_DISPATCHLOAD": "SETTLEMENTDATE",
    "INTERMITTENT_GEN_SCADA": "RUN_DATETIME",
    "TRADINGLOAD": "SETTLEMENTDATE",
    "TRADINGPRICE": "SETTLEMENTDATE",
    "TRADINGREGIONSUM": "SETTLEMENTDATE",
    "TRADINGINTERCONNECT": "SETTLEMENTDATE",
    "DUDETAILSUMMARY": "START_DATE",
    "PARTICIPANT": "LASTCHANGED",
    "DUDETAIL": "EFFECTIVEDATE",
    "DISPATCHCONSTRAINT": "SETTLEMENTDATE",
    "GENCONDATA": "EFFECTIVEDATE",
    "DISPATCH_UNIT_SCADA": "SETTLEMENTDATE",
    "DISPATCHPRICE": "SETTLEMENTDATE",
    "SPDREGIONCONSTRAINT": "EFFECTIVEDATE",
    "SPDCONNECTIONPOINTCONSTRAINT": "EFFECTIVEDATE",
    "SPDINTERCONNECTORCONSTRAINT": "EFFECTIVEDATE",
    "BIDPEROFFER_D": "INTERVAL_DATETIME",
    "DISPATCHINTERCONNECTORRES": "SETTLEMENTDATE",
    "BIDDAYOFFER_D": "SETTLEMENTDATE",
    "DISPATCHREGIONSUM": "SETTLEMENTDATE",
    "FCAS_4_SECOND": "TIMESTAMP",
    "ELEMENTS_FCAS_4_SECOND": None,
    "VARIABLES_FCAS_4_SECOND": None,
    "Generators and Scheduled Loads": None,
    "FCAS Providers": None,
    "MNSP_INTERCONNECTOR": "EFFECTIVEDATE",
    "MNSP_PEROFFER": "SETTLEMENTDATE",
    "INTERCONNECTOR": "LASTCHANGED",
    "INTERCONNECTORCONSTRAINT": "EFFECTIVEDATE",
    "MNSP_DAYOFFER": "SETTLEMENTDATE",
    "LOSSMODEL": "EFFECTIVEDATE",
    "LOSSFACTORMODEL": "EFFECTIVEDATE",
    "FCAS_4s_SCADA_MAP": None,
    "MARKET_PRICE_THRESHOLDS": "EFFECTIVEDATE",
    "DAILY_REGION_SUMMARY": "SETTLEMENTDATE",
    "ROOFTOP_PV_ACTUAL": "INTERVAL_DATETIME"
}

reg_exemption_list_tabs = {
    "Generators and Scheduled Loads": ["Generators and Scheduled Loads", "PU and Scheduled Loads"],
    "FCAS Providers": ["Ancillary Services"],
}

months = ["01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12"]

date_formats = [
    "%Y/%m/%d %H:%M:%S",
    '%Y/%m/%d %H:%M:%S.%f',
    '%Y-%m-%d %H:%M:%S'
]

nem_data_model_start_time = "2009/07/01 00:00:00"


# GUI settings.
header_y_pad = 30
query_y_pad = (20, 0)
query_row_offset = 2
row_height = 6
names_internal_row = 1
table_list_internal_row = 1
start_time_label_internal_row = 2
start_time_internal_row = 3
end_time_label_internal_row = 4
end_time_internal_row = 5
plus_internal_row = 6
plus_merge_internal_row = 7
list_row_span = 5
list_column_span = 1
save_field_column_span = 3
standard_x_pad = (0, 10)
list_filter_row_span = 4
internal_filter_row = 2
delete_button_internal_row = 5
last_column = 100
join_type = ["inner", "left", "right"]

# Testing settings
raw_data_cache = "/media/nick/Samsung_T5/nemosis_test_cache"



import unittest
import pandas as pd
from src.nemosis import custom_tables, defaults, data_fetch_methods
import math
import numpy as np
import time
from datetime import datetime, timedelta
import os


class TestBaseVolumeWeightAveragePriceFunction(unittest.TestCase):
    def setUp(self):
        self.volume = pd.Series([55, 0, math.nan, 60, 40])
        self.price = pd.Series([88, 90, -100, -100, 50])
        self.pricing_data = pd.DataFrame(
            {
                "SCADAVALUE": [8, 9, 50, 11, 10, 0],
                "TRADING_TOTALCLEARED": [
                    20,
                    math.nan,
                    math.nan,
                    80,
                    math.nan,
                    math.nan,
                ],
                "TRADING_RRP": [80, math.nan, math.nan, 100, math.nan, math.nan],
                "DISPATCH_RRP": [80, 90, 89, 111, 110, 75],
            }
        )
        pass

    def test_volume_weighted_average_price(self):
        vwap = custom_tables.volume_weighted_average_price(self.volume, self.price)
        self.assertAlmostEqual(vwap, 5.419354839, 4)

    def test_trading_price(self):
        vwap = custom_tables.volume_weighted_average_trading_price(self.pricing_data)
        self.assertAlmostEqual(vwap, 96, 4)

    def test_spot_price(self):
        vwap = custom_tables.volume_weighted_average_spot_price(self.pricing_data)
        self.assertAlmostEqual(vwap, 93.42045455, 4)


class TestPerformanceAtNodalPeak(unittest.TestCase):
    def setUp(self):
        self.cap_and_output = pd.DataFrame(
            {
                "SETTLEMENTDATE": [
                    "2015/01/01 00:00:00",
                    "2015/01/01 00:05:00",
                    "2015/01/01 00:10:00",
                    "2015/01/01 12:00:00",
                    "2015/01/01 12:10:00",
                    "2015/01/01 22:10:00",
                ],
                "DUID": ["A", "A", "A", "A", "A", "A"],
                "MAXCAPACITY": [68, 68, 68, 68, 68, 68],
                "SCADAVALUE": [8, 9, 50, 11, 10, 0],
                "TOTALDEMAND": [1000, 1100, 13000, 900, 800, 1000],
            }
        )
        self.cap_and_output["SETTLEMENTDATE"] = pd.to_datetime(
            self.cap_and_output["SETTLEMENTDATE"]
        )
        self.cap_and_output_nans = pd.DataFrame(
            {
                "SETTLEMENTDATE": [
                    "2015/01/01 00:00:00",
                    "2015/01/01 00:05:00",
                    "2015/01/01 00:10:00",
                    "2015/01/01 12:00:00",
                    "2015/01/01 12:10:00",
                    "2015/01/01 22:10:00",
                ],
                "DUID": ["A", "A", "A", "A", "A", "A"],
                "MAXCAPACITY": [68, 68, 68, 68, 68, 68],
                "SCADAVALUE": [8, 9, math.nan, 11, 10, 0],
                "TOTALDEMAND": [1000, 1100, 13000, 900, 800, 1000],
            }
        )
        self.cap_and_output_nans["SETTLEMENTDATE"] = pd.to_datetime(
            self.cap_and_output["SETTLEMENTDATE"]
        )

        pass

    def test_performance_at_nodal_peak(self):
        peak = custom_tables.performance_at_nodal_peak(self.cap_and_output)
        self.assertAlmostEqual(peak, 50 / 68, 4)

    def test_performance_at_nodal_peak_nans(self):
        peak = custom_tables.performance_at_nodal_peak(self.cap_and_output_nans)
        self.assertAlmostEqual(peak, 0.0, 4)


class TestCapacityFactor90(unittest.TestCase):
    def setUp(self):
        self.cap_and_output = pd.DataFrame(
            {
                "SETTLEMENTDATE": [
                    "2015/01/01 00:00:00",
                    "2015/01/01 00:05:00",
                    "2015/01/01 00:10:00",
                    "2015/01/01 12:00:00",
                    "2015/01/01 12:10:00",
                    "2015/01/01 22:10:00",
                ],
                "DUID": ["A", "A", "A", "A", "A", "A"],
                "MAXCAPACITY": [68, 68, 68, 68, 68, 68],
                "SCADAVALUE": [8, 9, 50, 11, 10, 0],
                "TOTALDEMAND": [1000, 1100, 13000, 900, 800, 1000],
            }
        )
        self.cap_and_output["SETTLEMENTDATE"] = pd.to_datetime(
            self.cap_and_output["SETTLEMENTDATE"]
        )
        self.cap_and_output2 = pd.DataFrame(
            {
                "SETTLEMENTDATE": [
                    "2015/01/01 00:00:00",
                    "2015/01/01 00:05:00",
                    "2015/01/01 00:10:00",
                    "2015/01/01 12:00:00",
                    "2015/01/01 12:10:00",
                    "2015/01/01 22:10:00",
                    "2015/01/02 00:00:00",
                    "2015/01/02 00:05:00",
                    "2015/01/02 00:10:00",
                    "2015/01/02 12:00:00",
                    "2015/01/02 12:10:00",
                    "2015/01/02 22:10:00",
                ],
                "DUID": ["A", "A", "A", "A", "A", "A", "A", "A", "A", "A", "A", "A"],
                "MAXCAPACITY": [68, 68, 68, 68, 68, 68, 68, 68, 68, 68, 68, 68],
                "SCADAVALUE": [8, 9, 50, 11, 10, 0, 8, 9, 50, 11, 10, 90],
                "TOTALDEMAND": [
                    10000,
                    11000,
                    13000,
                    900,
                    800,
                    1000,
                    1000,
                    11000,
                    13000,
                    900,
                    800,
                    18000,
                ],
            }
        )
        self.cap_and_output2["SETTLEMENTDATE"] = pd.to_datetime(
            self.cap_and_output2["SETTLEMENTDATE"]
        )
        self.cap_and_output2_nans = pd.DataFrame(
            {
                "SETTLEMENTDATE": [
                    "2015/01/01 00:00:00",
                    "2015/01/01 00:05:00",
                    "2015/01/01 00:10:00",
                    "2015/01/01 12:00:00",
                    "2015/01/01 12:10:00",
                    "2015/01/01 22:10:00",
                    "2015/01/02 00:00:00",
                    "2015/01/02 00:05:00",
                    "2015/01/02 00:10:00",
                    "2015/01/02 12:00:00",
                    "2015/01/02 12:10:00",
                    "2015/01/02 22:10:00",
                ],
                "DUID": ["A", "A", "A", "A", "A", "A", "A", "A", "A", "A", "A", "A"],
                "MAXCAPACITY": [68, 68, 68, 68, 68, 68, 68, 68, 68, 68, 68, 68],
                "SCADAVALUE": [8, 9, 50, 11, 10, 0, 8, 9, 50, 11, 10, math.nan],
                "TOTALDEMAND": [
                    10000,
                    11000,
                    13000,
                    900,
                    800,
                    1000,
                    1000,
                    11000,
                    13000,
                    900,
                    800,
                    18000,
                ],
            }
        )
        self.cap_and_output2_nans["SETTLEMENTDATE"] = pd.to_datetime(
            self.cap_and_output2["SETTLEMENTDATE"]
        )
        self.cap_and_output2_nans2 = pd.DataFrame(
            {
                "SETTLEMENTDATE": [
                    "2015/01/01 00:00:00",
                    "2015/01/01 00:05:00",
                    "2015/01/01 00:10:00",
                    "2015/01/01 12:00:00",
                    "2015/01/01 12:10:00",
                    "2015/01/01 22:10:00",
                    "2015/01/02 00:00:00",
                    "2015/01/02 00:05:00",
                    "2015/01/02 00:10:00",
                    "2015/01/02 12:00:00",
                    "2015/01/02 12:10:00",
                    "2015/01/02 22:10:00",
                ],
                "DUID": ["A", "A", "A", "A", "A", "A", "A", "A", "A", "A", "A", "A"],
                "MAXCAPACITY": [
                    math.nan,
                    math.nan,
                    math.nan,
                    68,
                    68,
                    68,
                    68,
                    68,
                    68,
                    68,
                    68,
                    68,
                ],
                "SCADAVALUE": [
                    math.nan,
                    math.nan,
                    math.nan,
                    11,
                    10,
                    0,
                    8,
                    9,
                    50,
                    11,
                    10,
                    90,
                ],
                "TOTALDEMAND": [
                    10000,
                    11000,
                    13000,
                    900,
                    800,
                    1000,
                    1000,
                    11000,
                    13000,
                    900,
                    800,
                    18000,
                ],
            }
        )
        self.cap_and_output2_nans2["SETTLEMENTDATE"] = pd.to_datetime(
            self.cap_and_output2["SETTLEMENTDATE"]
        )
        pass

    def test_capacity_factor(self):
        peak = custom_tables.capacity_factor_over_90th_percentile_of_nodal_demand(
            self.cap_and_output
        )
        self.assertAlmostEqual(peak, 50 / 68, 4)

    def test_capacity_factor_2_intervals(self):
        peak = custom_tables.capacity_factor_over_90th_percentile_of_nodal_demand(
            self.cap_and_output2
        )
        self.assertAlmostEqual(peak, (50 / 68 + 90 / 68) / 2, 4)

    def test_capacity_factor_2_intervals_one_nan(self):
        peak = custom_tables.capacity_factor_over_90th_percentile_of_nodal_demand(
            self.cap_and_output2_nans
        )
        self.assertAlmostEqual(peak, (50 / 68 + 0.0 / 68) / 2, 4)

    def test_capacity_factor_2_intervals_plant_built_after_first(self):
        peak = custom_tables.capacity_factor_over_90th_percentile_of_nodal_demand(
            self.cap_and_output2_nans2
        )
        self.assertAlmostEqual(peak, (90 / 68), 4)


class TestCapacityScadaBasedStats(unittest.TestCase):
    def setUp(self):
        self.cap_and_output = pd.DataFrame(
            {"MAXCAPACITY": [68, 68, 68, 68, 68], "SCADAVALUE": [8, 9, 50, 11, 10]}
        )
        pass

    def test_capacity_factor(self):
        cf = custom_tables.capacity_factor(self.cap_and_output)
        self.assertAlmostEqual(cf, 0.258823529, 4)

    def test_volume(self):
        v = custom_tables.volume(self.cap_and_output)
        self.assertAlmostEqual(v, 7.3333, 4)


class TestMonthlyGroupingForStats(unittest.TestCase):
    def setUp(self):
        self.cap_and_output = pd.DataFrame(
            {
                "SETTLEMENTDATE": [
                    "2015/01/01 00:00:00",
                    "2015/01/01 00:05:00",
                    "2015/01/01 00:10:00",
                    "2015/02/01 00:00:00",
                    "2015/02/01 00:05:00",
                    "2015/02/01 00:10:00",
                ],
                "DUID": ["A", "A", "A", "A", "A", "A"],
                "MAXCAPACITY": [68, 68, 68, 68, 68, 68],
                "SCADAVALUE": [8, 9, 50, 11, 10, 0],
                "TRADING_TOTALCLEARED": [
                    111,
                    math.nan,
                    math.nan,
                    250,
                    math.nan,
                    math.nan,
                ],
                "TRADING_RRP": [115, math.nan, math.nan, 250, math.nan, math.nan],
                "DISPATCH_RRP": [112, 97, 102, 81, 85, 91],
                "TOTALDEMAND": [1000, 1020, 1100, 990, 1100, 897],
            }
        )
        self.cap_and_output["SETTLEMENTDATE"] = pd.to_datetime(
            self.cap_and_output["SETTLEMENTDATE"]
        )
        self.cf_by_month = pd.DataFrame(
            {
                "Month": ["2015-01", "2015-02"],
                "DUID": ["A", "A"],
                "CapacityFactor": [0.328431373, 0.102941176],
                "Volume": [67.0 / 12, 21.0 / 12],
                "TRADING_VWAP": [115.0, 250.0],
                "DISPATCH_VWAP": [102.5223881, 82.9047619],
                "NodalPeakCapacityFactor": [0.735294118, 0.147058824],
                "Nodal90thPercentileCapacityFactor": [0.735294118, 0.147058824],
            }
        )
        pass

    def test_one_duid_two_months_example(self):
        cf_by_month_and_duid = custom_tables.stats_by_month_and_plant(
            self.cap_and_output
        )
        cf_by_month_and_duid = cf_by_month_and_duid.reset_index(drop=True)
        pd.testing.assert_frame_equal(cf_by_month_and_duid, self.cf_by_month)


class TestMergeTables(unittest.TestCase):
    def setUp(self):
        self.gen_info = pd.DataFrame(
            {
                "EFFECTIVEDATE": [
                    "2014/01/01 00:00:00",
                    "2017/01/01 00:00:00",
                    "2014/01/01 00:00:00",
                ],
                "DUID": ["A", "A", "B"],
                "MAXCAPACITY": [333, 400, 250],
            }
        )
        self.gen_info["EFFECTIVEDATE"] = pd.to_datetime(self.gen_info["EFFECTIVEDATE"])
        self.gen_info2 = pd.DataFrame(
            {
                "START_DATE": [
                    "2014/01/01 00:00:00",
                    "2017/01/01 00:00:00",
                    "2014/01/01 00:00:00",
                ],
                "DUID": ["A", "A", "B"],
                "REGIONID": ["NSW1", "NSW1", "VIC1"],
            }
        )
        self.gen_info2["START_DATE"] = pd.to_datetime(self.gen_info2["START_DATE"])
        self.scada = pd.DataFrame(
            {
                "DUID": ["A", "A", "B", "B"],
                "SETTLEMENTDATE": [
                    "2015/01/01 00:00:00",
                    "2015/01/01 00:05:00",
                    "2015/01/01 00:00:00",
                    "2015/01/01 00:05:00",
                ],
                "SCADAVALUE": [150, 150, 200, 220],
            }
        )
        self.scada["SETTLEMENTDATE"] = pd.to_datetime(self.scada["SETTLEMENTDATE"])
        self.trading_load = pd.DataFrame(
            {
                "DUID": ["A", "B"],
                "SETTLEMENTDATE": ["2015/01/01 00:00:00", "2015/01/01 00:00:00"],
                "TOTALCLEARED": [150, 200],
            }
        )
        self.trading_load["SETTLEMENTDATE"] = pd.to_datetime(
            self.trading_load["SETTLEMENTDATE"]
        )
        self.dispatch_price = pd.DataFrame(
            {
                "REGIONID": ["NSW1", "NSW1", "VIC1", "VIC1"],
                "SETTLEMENTDATE": [
                    "2015/01/01 00:00:00",
                    "2015/01/01 00:05:00",
                    "2015/01/01 00:00:00",
                    "2015/01/01 00:05:00",
                ],
                "RRP": [99, 110, 300, 500],
            }
        )
        self.dispatch_price["SETTLEMENTDATE"] = pd.to_datetime(
            self.dispatch_price["SETTLEMENTDATE"]
        )
        self.trading_price = pd.DataFrame(
            {
                "REGIONID": ["NSW1", "VIC1"],
                "SETTLEMENTDATE": ["2015/01/01 00:00:00", "2015/01/01 00:00:00"],
                "RRP": [99, 300],
            }
        )
        self.trading_price["SETTLEMENTDATE"] = pd.to_datetime(
            self.trading_price["SETTLEMENTDATE"]
        )
        self.region_summary = pd.DataFrame(
            {
                "REGIONID": ["NSW1", "NSW1", "VIC1", "VIC1"],
                "SETTLEMENTDATE": [
                    "2015/01/01 00:00:00",
                    "2015/01/01 00:05:00",
                    "2015/01/01 00:00:00",
                    "2015/01/01 00:05:00",
                ],
                "TOTALDEMAND": [5000, 5010, 8000, 8700],
            }
        )
        self.region_summary["SETTLEMENTDATE"] = pd.to_datetime(
            self.region_summary["SETTLEMENTDATE"]
        )

        self.expected_combined_df = pd.DataFrame(
            {
                "DUID": ["A", "A", "B", "B"],
                "DUDETAIL_EFFECTIVEDATE": [
                    "2014/01/01 00:00:00",
                    "2014/01/01 00:00:00",
                    "2014/01/01 00:00:00",
                    "2014/01/01 00:00:00",
                ],
                "REGIONID": ["NSW1", "NSW1", "VIC1", "VIC1"],
                "SETTLEMENTDATE": [
                    "2015/01/01 00:00:00",
                    "2015/01/01 00:05:00",
                    "2015/01/01 00:00:00",
                    "2015/01/01 00:05:00",
                ],
                "MAXCAPACITY": [333, 333, 250, 250],
                "SCADAVALUE": [150, 150, 200, 220],
                "TRADING_TOTALCLEARED": [150, math.nan, 200, math.nan],
                "DISPATCH_RRP": [99, 110, 300, 500],
                "TRADING_RRP": [99, math.nan, 300, math.nan],
                "TOTALDEMAND": [5000, 5010, 8000, 8700],
            }
        )
        ix = pd.date_range(
            start=datetime.strptime("2015/01/01 00:00:00", "%Y/%m/%d %H:%M:%S"),
            end=datetime.strptime("2015/01/01 00:10:00", "%Y/%m/%d %H:%M:%S")
            - timedelta(minutes=5),
            freq="5T",
        )
        self.timeseries_df = pd.DataFrame(index=ix)
        self.timeseries_df.reset_index(inplace=True)
        self.timeseries_df.columns = ["SETTLEMENTDATE"]
        self.expected_combined_df["SETTLEMENTDATE"] = pd.to_datetime(
            self.expected_combined_df["SETTLEMENTDATE"]
        )
        self.expected_combined_df = self.expected_combined_df.sort_values(
            "SETTLEMENTDATE"
        )

    def test_merge_tables(self):
        merged_table = custom_tables.merge_tables_for_plant_stats(
            self.timeseries_df,
            self.gen_info,
            self.gen_info2,
            self.scada,
            self.trading_load,
            self.dispatch_price,
            self.trading_price,
            self.region_summary,
        )
        np.array_equal(merged_table, self.expected_combined_df)


class TestSelectHighestVersionNumber(unittest.TestCase):
    def setUp(self):
        self.gen_info = pd.DataFrame(
            {
                "EFFECTIVEDATE": [
                    "2014/01/01 00:00:00",
                    "2017/01/01 00:00:00",
                    "2014/01/01 00:00:00",
                    "2014/01/01 00:00:00",
                ],
                "VERSIONNO": ["1", "1", "1", "2"],
                "DUID": ["A", "A", "B", "B"],
                "MAXCAPACITY": [333, 400, 250, 800],
            }
        )
        self.gen_info["EFFECTIVEDATE"] = pd.to_datetime(self.gen_info["EFFECTIVEDATE"])
        self.expected_result = pd.DataFrame(
            {
                "EFFECTIVEDATE": [
                    "2014/01/01 00:00:00",
                    "2017/01/01 00:00:00",
                    "2014/01/01 00:00:00",
                ],
                "VERSIONNO": ["1", "1", "2"],
                "DUID": ["A", "A", "B"],
                "MAXCAPACITY": [333, 400, 800],
            }
        )
        self.expected_result["EFFECTIVEDATE"] = pd.to_datetime(
            self.expected_result["EFFECTIVEDATE"]
        )

    def test_select_highest_version_no(self):
        result = custom_tables.select_highest_version_number(
            self.gen_info, defaults.table_primary_keys["DUDETAIL"]
        )
        np.array_equal(result, self.expected_result)


class TestSelectInterventionIfPresent(unittest.TestCase):
    def setUp(self):
        self.dispatch_price = pd.DataFrame(
            {
                "REGIONID": ["NSW1", "NSW1", "NSW1", "VIC1", "VIC1"],
                "SETTLEMENTDATE": [
                    "2015/01/01 00:00:00",
                    "2015/01/01 00:05:00",
                    "2015/01/01 00:05:00",
                    "2015/01/01 00:00:00",
                    "2015/01/01 00:05:00",
                ],
                "RRP": [99, 110, 112, 300, 500],
                "INTERVENTION": ["0", "0", "1", "0", "1"],
            }
        )
        self.dispatch_price["SETTLEMENTDATE"] = pd.to_datetime(
            self.dispatch_price["SETTLEMENTDATE"]
        )
        self.expected_result = pd.DataFrame(
            {
                "REGIONID": ["NSW1", "NSW1", "VIC1", "VIC1"],
                "SETTLEMENTDATE": [
                    "2015/01/01 00:00:00",
                    "2015/01/01 00:05:00",
                    "2015/01/01 00:00:00",
                    "2015/01/01 00:05:00",
                ],
                "RRP": [99, 112, 300, 500],
                "INTERVENTION": ["0", "1", "0", "1"],
            }
        )
        self.expected_result["SETTLEMENTDATE"] = pd.to_datetime(
            self.expected_result["SETTLEMENTDATE"]
        )

    def test_select_highest_version_no(self):
        result = custom_tables.select_intervention_if_present(
            self.dispatch_price, defaults.table_primary_keys["DISPATCHPRICE"]
        )
        np.array_equal(result, self.expected_result)


class TestPlantStats(unittest.TestCase):
    def setUp(self):
        pass

    def test_plant_stats(self):
        if os.path.isfile("C:/Users/user/Documents/plant_stats.csv"):
            t0 = time.time()
            plant_types = data_fetch_methods.static_table_xl(
                "",
                "",
                "Generators and Scheduled Loads",
                defaults.raw_data_cache,
                select_columns=[
                    "DUID",
                    "Fuel Source - Primary",
                    "Region",
                    "Participant",
                ],
            )
            plant_stats = custom_tables.plant_stats(
                "2017/07/01 00:05:00",
                "2018/07/01 00:05:00",
                "",
                defaults.raw_data_cache,
            )
            plant_stats = pd.merge(plant_stats, plant_types, "left", "DUID")
            plant_stats["TRADING_COST"] = (
                plant_stats["Volume"] * plant_stats["TRADING_VWAP"]
            )
            plant_stats["DISPATCH_COST"] = (
                plant_stats["Volume"] * plant_stats["DISPATCH_VWAP"]
            )
            plant_stats.to_csv("C:/Users/user/Documents/plant_stats_tp.csv")
            print(time.time() - t0)


class TestPlantsAgainstExcelNumbers(unittest.TestCase):
    def setUp(self):
        pass

    def test_nyngan1(self):
        if os.path.isfile("E:/plants_stats_test_data/NYNGAN1/NYNGAN1_test.xlsx"):
            xls = pd.ExcelFile("E:/plants_stats_test_data/NYNGAN1/NYNGAN1_test.xlsx")
            table = pd.read_excel(xls, "Plant_stats", dtype=str)
            results = custom_tables.plant_stats(
                "2017/01/01 00:00:00",
                "2017/02/01 00:00:00",
                "",
                defaults.raw_data_cache,
                filter_cols=["DUID"],
                filter_values=[("NYNGAN1",)],
            )
            for col in [col for col in table.columns if col not in ["Month", "DUID"]]:
                table[col] = table[col].astype(float)
            results.reset_index(drop=True, inplace=True)
            pd.testing.assert_frame_equal(results, table)

    def test_eildon2(self):
        if os.path.isfile("E:/plants_stats_test_data/EILDON2/EILDON2_test.xlsx"):
            xls = pd.ExcelFile("E:/plants_stats_test_data/EILDON2/EILDON2_test.xlsx")
            table = pd.read_excel(xls, "Plant_stats", dtype=str)
            results = custom_tables.plant_stats(
                "2018/01/01 00:00:00",
                "2018/02/01 00:00:00",
                "",
                defaults.raw_data_cache,
                filter_cols=["DUID"],
                filter_values=[("EILDON2",)],
            )
            for col in [col for col in table.columns if col not in ["Month", "DUID"]]:
                table[col] = table[col].astype(float)
            results.reset_index(drop=True, inplace=True)
            pd.testing.assert_frame_equal(results, table)


class TestCalcTradingLoad(unittest.TestCase):
    def setUp(self):
        self.scada = pd.DataFrame(
            {
                "DUID": ["A", "A", "B", "B"],
                "SETTLEMENTDATE": [
                    "2015/01/01 00:00:00",
                    "2015/01/01 00:05:00",
                    "2015/01/01 00:00:00",
                    "2015/01/01 00:05:00",
                ],
                "SCADAVALUE": [150, 150, 200, 220],
            }
        )
        self.scada["SETTLEMENTDATE"] = pd.to_datetime(self.scada["SETTLEMENTDATE"])
        self.result = pd.DataFrame(
            {
                "DUID": ["A", "A", "B", "B"],
                "TOTALCLEARED": [150, 150, 200, 220],
                "SETTLEMENTDATE": [
                    "2015/01/01 00:00:00",
                    "2015/01/01 00:05:00",
                    "2015/01/01 00:00:00",
                    "2015/01/01 00:05:00",
                ],
            }
        )
        self.result["SETTLEMENTDATE"] = pd.to_datetime(self.result["SETTLEMENTDATE"])

    def test_calc_trading_load_simple(self):
        trading_load = custom_tables.calc_trading_load(self.scada)
        pd.testing.assert_frame_equal(trading_load, self.result)


class TestCalcTradingLoad(unittest.TestCase):
    def setUp(self):
        pass

    def test_calc_trading_load_simple(self):
        pass
        # custom_tables.trading_and_dispatch_cost()



import os
from nemosis import defaults


def write_mms_file_names(name, month, year, day, chunk, index, raw_data_location):
    # Add the year and month information to the generic AEMO file name
    if int(year) > 2024 or (int(year) == 2024 and int(month) >= 8):
        filename_stub = (
                'PUBLIC_ARCHIVE#' + name + "#FILE" + str(chunk).zfill(2) + '#' + str(year) + str(month) + "010000"
        )
    else:
        filename_stub = defaults.names[name] + "_" + str(year) + str(month) + "010000"
    path_and_name = os.path.join(raw_data_location, filename_stub)
    return filename_stub, path_and_name


def write_file_names_mms_and_current(name, month, year, day, chunk, index, raw_data_location):
    if day is None:
        filename_stub, path_and_name = write_mms_file_names(name, month, year, day, index, chunk, raw_data_location)
    else:
        filename_stub = (
                defaults.names[name] + "_" + str(year) + str(month) + str(day)
        )
        path_and_name = os.path.join(raw_data_location, filename_stub)
    return filename_stub, path_and_name


def write_file_names_current(name, month, year, day, chunk, index, raw_data_location):
    # Add the year and month information to the generic AEMO file name
    filename_stub = (
            defaults.names[name] + "_" + str(year) + str(month) + str(day)
    )
    path_and_name = os.path.join(raw_data_location, filename_stub)
    return filename_stub, path_and_name


def write_file_names_fcas(name, month, year, day, chunk, index, raw_data_location):
    # Add the year and month information to the generic AEMO file name
    filename_stub = defaults.names[name] + "_" + str(year) + str(month) + day + index
    path_and_name = os.path.join(raw_data_location, filename_stub)
    return filename_stub, path_and_name


import logging
from datetime import datetime, timedelta
import numpy as np

from nemosis.value_parser import _parse_datetime

logger = logging.getLogger(__name__)

def filter_on_start_and_end_date(data, start_time, end_time):
    data["START_DATE"] = _parse_datetime(data["START_DATE"])
    data["END_DATE"] = np.where(
        data["END_DATE"] == "2999/12/31 00:00:00",
        "2100/12/31 00:00:00",
        data["END_DATE"],
    )
    data["END_DATE"] = _parse_datetime(data["END_DATE"])
    data = data[(data["START_DATE"] < end_time) & (data["END_DATE"] > start_time)]
    return data


def filter_on_effective_date(data, start_time, end_time):
    data["EFFECTIVEDATE"] = _parse_datetime(data["EFFECTIVEDATE"])
    data = data[data["EFFECTIVEDATE"] < end_time]
    return data


def filter_on_settlementdate(data, start_time, end_time):
    data["SETTLEMENTDATE"] = _parse_datetime(data["SETTLEMENTDATE"])
    data = data[
        (data["SETTLEMENTDATE"] > start_time) & (data["SETTLEMENTDATE"] <= end_time)
    ]
    return data


def filter_on_run_datetime(data, start_time, end_time):
    data["RUN_DATETIME"] = _parse_datetime(data["RUN_DATETIME"])
    data = data[
        (data["RUN_DATETIME"] > start_time) & (data["RUN_DATETIME"] <= end_time)
    ]
    return data


def filter_on_timestamp(data, start_time, end_time):
    try:
        data["TIMESTAMP"] = _parse_datetime(data["TIMESTAMP"])
    except ValueError as e:
        logger.error(e)
        # if date format is wrong, str may be too short
        med_str_len = np.median(data["TIMESTAMP"].str.len())
        not_data = data.loc[data["TIMESTAMP"].str.len() < med_str_len, :]
        data = data.loc[data["TIMESTAMP"].str.len() >= med_str_len, :]
        data["TIMESTAMP"] = _parse_datetime(data["TIMESTAMP"])
        logger.warning("Rows with incorrect data formats omitted")
        logger.warning(not_data.head())
    finally:
        data = data[(data["TIMESTAMP"] > start_time) & (data["TIMESTAMP"] <= end_time)]
    return data


def filter_on_interval_datetime(data, start_time, end_time):
    data["INTERVAL_DATETIME"] = _parse_datetime(data["INTERVAL_DATETIME"])
    data = data[
        (data["INTERVAL_DATETIME"] > start_time)
        & (data["INTERVAL_DATETIME"] <= end_time)
    ]
    return data


# Not tested, just for nemlite integration.
def filter_on_date_and_peroid(data, start_time, end_time):
    data = construct_interval_datetime_from_period_id(data)
    data = data[
        (data["SETTLEMENTDATE"] > start_time)
        & (data["SETTLEMENTDATE"] - timedelta(minutes=30) <= end_time)
    ]
    return data


# Not tested, just for nemlite integration.
def filter_on_date_and_interval(data, start_time, end_time):
    data["SETTLEMENTDATE"] = _parse_datetime(data["SETTLEMENTDATE"])
    data = data[
        (data["SETTLEMENTDATE"] > start_time) & (data["SETTLEMENTDATE"] <= end_time)
    ]
    return data


# Not tested, just for nemlite integration.
def filter_on_last_changed(data, start_time, end_time):
    data["LASTCHANGED"] = _parse_datetime(data["LASTCHANGED"])
    data = data[data["LASTCHANGED"] < end_time]
    return data


def filter_on_column_value(data, filter_cols, filter_values):
    for filter_col, filter_values in zip(filter_cols, filter_values):
        if filter_values is not None:
            data = data[data[filter_col].isin(filter_values)]
    return data


# Not tested, just for nemlite integration.
def construct_interval_datetime_from_period_id(data):
    data["SETTLEMENTDATE"] = np.vectorize(date_2_interval_datetime)(
        data["SETTLEMENTDATE"], data["PERIODID"]
    )
    return data


# Not tested, just for nemlite integration.
def date_2_interval_datetime(date, period):
    datetime_obj = datetime.strptime(date, "%Y/%m/%d %H:%M:%S")
    datetime_obj = datetime(
        year=datetime_obj.year, month=datetime_obj.month, day=datetime_obj.day, hour=4
    )
    datetime_obj = datetime_obj + timedelta(minutes=(float(period) * 30))
    return datetime_obj



import pandas as pd
import numpy as np
from datetime import timedelta
from src.nemosis import static_table, dynamic_data_compiler
import plotly.express as px

# Specify where we will be caching the raw AEMO data.
raw_data_cache = "C:/Users/nick/Desktop/cache"

# Time window to pull data from.
start_time = "2021/04/27 00:00:00"
end_time = "2021/04/28 00:00:00"

# Download the latest FCAS causer pays elements file. The update_static_file=True argument forces nemosis to
# download the a new copy of file from AEMO even if a copy already exists in the cache.
fcas_causer_pays_elements = static_table(
    table_name="ELEMENTS_FCAS_4_SECOND",
    raw_data_location=raw_data_cache,
    update_static_file=True,
)

# Using filtering and manual inspection find which fcas element numbers belong to Hornsdale Power Reserve.
elements_for_honsdale_power_reserve = fcas_causer_pays_elements[
    fcas_causer_pays_elements["EMSNAME"].str.contains("HPR")
]

# Check which variable numbers we will need.
fcas_causer_pays_elements = static_table(
    table_name="ELEMENTS_FCAS_4_SECOND",
    raw_data_location=raw_data_cache,
    update_static_file=True,
)

scada_4s_resolution = dynamic_data_compiler(
    start_time,
    end_time,
    table_name="FCAS_4_SECOND",
    raw_data_location=raw_data_cache,
    filter_cols=["ELEMENTNUMBER", "VARIABLENUMBER"],
    filter_values=([330, 331], [2, 5]),
    fformat="parquet",
)

scada_5min_resolution = dynamic_data_compiler(
    start_time,
    end_time,
    "DISPATCHLOAD",
    raw_data_cache,
    select_columns=["SETTLEMENTDATE", "DUID", "INITIALMW", "TOTALCLEARED"],
    filter_cols=["DUID"],
    filter_values=(["HPRG1", "HPRL1"],),
)

elements = {330: "HPRG1", 331: "HPRL1"}

variables = {2: "scada_value", 5: "regulation_target"}

scada_4s_resolution["DUID"] = scada_4s_resolution["ELEMENTNUMBER"].apply(
    lambda x: elements[x]
)
scada_4s_resolution["variable"] = scada_4s_resolution["VARIABLENUMBER"].apply(
    lambda x: variables[x]
)

scada_4s_resolution = scada_4s_resolution.pivot(
    index=["TIMESTAMP", "DUID"], columns="variable", values="VALUE"
)

scada_4s_resolution.reset_index(inplace=True)

scada = pd.merge_asof(
    scada_4s_resolution,
    scada_5min_resolution,
    left_on="TIMESTAMP",
    right_on="SETTLEMENTDATE",
    by="DUID",
    direction="forward",
)

scada["fraction_ramp_complete"] = 1 - (
    (scada["SETTLEMENTDATE"] - scada["TIMESTAMP"]) / timedelta(minutes=5)
)

scada["linear_ramp_target"] = (
    scada["INITIALMW"]
    + (scada["TOTALCLEARED"] - scada["INITIALMW"]) * scada["fraction_ramp_complete"]
)

scada["linear_ramp_target"] = np.where(
    scada["DUID"] == "HPRL1",
    -1 * scada["linear_ramp_target"],
    scada["linear_ramp_target"],
)
scada["scada_value"] = np.where(
    scada["DUID"] == "HPRL1", -1 * scada["scada_value"], scada["scada_value"]
)
scada["regulation_target"] = np.where(
    scada["DUID"] == "HPRL1",
    -1 * scada["regulation_target"],
    scada["regulation_target"],
)

scada = scada.groupby("TIMESTAMP", as_index=False).agg(
    {"linear_ramp_target": "sum", "scada_value": "sum", "regulation_target": "sum"}
)

scada["target"] = scada["linear_ramp_target"] + scada["regulation_target"]

fig = px.line(scada, x="TIMESTAMP", y=["target", "scada_value"])
fig.show()
